import 'dart:io';

import 'interfaces.dart';
import 'request.dart';
import 'response.dart';

typedef Conn = HttpClient Function();

class Client {
  final List<RoundTripBuilder> roundTripBuilders;
  late Conn conn;

  Client({
    Conn? conn,
    this.roundTripBuilders = const [],
  }) {
    this.conn = conn ?? () => HttpClient();
  }

  Future<Response> fetch(Request request) {
    RoundTrip rt = _send;
    for (var n = roundTripBuilders.length - 1; n >= 0; n--) {
      rt = roundTripBuilders[n].build(rt);
    }
    return rt(request);
  }

  Future<Response> _send(Request request) async {
    var c = conn();

    request.cancelToken?.whenCancel.then((cancelErr) {
      try {
        c.close(force: true);
      } catch (e) {
        //...
      }
    });

    try {
      var req = await c.openUrl(
        request.method,
        request.uri,
      );

      request.headers?.forEach((key, value) => req.headers.set(key, value));

      if (request.requestBody != null) {
        await req.addStream(request.requestBody!);
      }

      var resp = await req.close();

      return Response(
        statusCode: resp.statusCode,
        headers: resp.headers,
        responseBody: resp,
        contentLength: resp.contentLength,
        request: request,
      );
    } on SocketException catch (e) {
      if (e.message.contains('cancelled')) {
        var cancelError = request.cancelToken?.cancelError;
        if (cancelError != null) {
          throw cancelError;
        }
      }
      rethrow;
    } finally {
      c.close();
    }
  }
}
