import 'dart:convert';
import 'dart:io';

import 'request.dart';

class Response<T> {
  final Request request;
  final int statusCode;
  final HttpHeaders headers;
  final int contentLength;
  final Stream<List<int>> responseBody;

  T? body;

  Response({
    required this.statusCode,
    required this.headers,
    required this.responseBody,
    required this.contentLength,
    required this.request,
  });

  String? _raw;

  Future<String> text() async {
    _raw ??= await responseBody.transform(utf8.decoder).join();
    return _raw!;
  }

  Future<dynamic> json() async {
    _raw ??= await responseBody.transform(utf8.decoder).join();
    body ??= _raw!.isNotEmpty ? jsonDecode(_raw!) : (<String, dynamic>{});
    return body;
  }
}

class ErrorResponse<T> {
  final int statusCode;
  final String? reason;
  final Response<T>? response;

  ErrorResponse(
    this.statusCode, {
    this.response,
    this.reason,
  });

  @override
  String toString() {
    return response?.body?.toString() ?? "$statusCode";
  }
}
